pub struct ZKWrappedProverChannel<'a, P, NTT, Channel, ReplayFn, A>where
P: PackedField<Scalar: BinaryField>,
NTT: AdditiveNTT<Field = P::Scalar> + Sync,
Channel: MerkleIPProverChannel<P::Scalar>,
A: Allocator,{ /* private fields */ }Expand description
A prover channel that wraps a BaseFoldProverChannel and an outer Spartan IOP prover.
This channel records all channel values. On
send_*/sample/observe_*, it delegates to the inner BaseFold channel and records each
value. After the inner proof is run through this channel, call
finish to replay the interaction, fill the outer witness, and generate the
outer proof.
The ReplayFn closure is called during finish with a ReplayChannel to
replay the inner verification and fill the outer witness. This allows the channel to be generic
over different inner verification protocols.
Implementations
Sourceimpl<'a, F, P, NTT, Channel, ReplayFn, A> ZKWrappedProverChannel<'a, P, NTT, Channel, ReplayFn, A>where
F: BinaryField,
P: PackedField<Scalar = F>,
NTT: AdditiveNTT<Field = F> + Sync,
Channel: MerkleIPProverChannel<F>,
A: Allocator,
impl<'a, F, P, NTT, Channel, ReplayFn, A> ZKWrappedProverChannel<'a, P, NTT, Channel, ReplayFn, A>where
F: BinaryField,
P: PackedField<Scalar = F>,
NTT: AdditiveNTT<Field = F> + Sync,
Channel: MerkleIPProverChannel<F>,
A: Allocator,
Sourcepub fn new(
inner_channel: BaseFoldProverChannel<'a, F, P, NTT, Channel, A>,
outer_prover: &'a IOPProver<F>,
outer_layout: Arc<WitnessLayout<F>>,
alloc: &'a A,
rng: impl CryptoRng,
replay_fn: ReplayFn,
) -> Self
pub fn new( inner_channel: BaseFoldProverChannel<'a, F, P, NTT, Channel, A>, outer_prover: &'a IOPProver<F>, outer_layout: Arc<WitnessLayout<F>>, alloc: &'a A, rng: impl CryptoRng, replay_fn: ReplayFn, ) -> Self
Creates a new ZK-wrapped prover channel.
Commits the outer prover’s precommit oracle on the inner channel as part of construction:
a random FieldBuffer<P> the size of the outer precommit oracle
segment is sent to the channel and kept for use in Self::finish. This random buffer is
the one-time-pad encryption key for the (future) outer encrypted transcript.
The inner channel’s oracle specs are expected to be laid out as
[outer_precommit, inner..., outer_private, outer_mask].
Arguments
inner_channel- The BaseFold ZK channel with oracle specs for both inner and outer proofsouter_prover- The IOP prover for the outer (wrapper) constraint systemouter_layout- The witness layout for the outer constraint systemalloc- Allocator for the outer proof’s working buffers, borrowed from the owning proverrng- RNG used to generate the random precommit buffer (the future OTP key)replay_fn- Closure called duringfinishwith aReplayChannelto replay the inner verification and fill the outer witness
Sourcepub fn finish(self, rng: impl CryptoRng) -> Result<(), Error>where
ReplayFn: FnOnce(&mut ReplayChannel<F>),
pub fn finish(self, rng: impl CryptoRng) -> Result<(), Error>where
ReplayFn: FnOnce(&mut ReplayChannel<F>),
Consumes the channel and runs the outer proof.
This should be called after the inner proof has been run through this channel. It:
- Creates a
ReplayChannelfrom the recorded interaction - Calls the
replay_fnclosure to replay the inner verification and fill the outer witness - Validates and generates the outer IOP proof